package com.gyee.redis2taos.config;

import com.alibaba.druid.pool.DruidDataSource;
import com.gyee.redis2taos.util.TaosDatabaseInfo;
import com.taosdata.jdbc.TSDBDriver;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.util.StringUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.util.Properties;

@Slf4j
@Configuration
public class TaosConfig {

    @Value("${taoscz.server_ip:192.168.1.60}")
    private String serverIp;

    @Value("${taoscz.server_port:6030}")
    private int serverPort;

    @Value("${taoscz.db_name:power_test}")
    private String dbName;

    @Value("${taoscz.user_name:root}")
    private String userName;

    @Value("${taoscz.password:taosdata}")
    private String password;

    @Value("${taoscz.pool_size:10}")
    private int poolSize;

    @Value("${taoscz.max_pool_size:100}")
    private int maxPoolSize;

    @Value("${taoscz.driver_type:com.taosdata.jdbc.rs.RestfulDriver}")
    private String driverType;

    @Value("${taoscz.ai_stable_name:windturbineai}")
    private String aiStableName;

    @Value("${taoscz.di_stable_name:windturbinedi}")
    private String diStableName;

    private Connection connection = null;
    private Connection createCon = null;
    private DruidDataSource dataSource = null;

    public Connection getInstance() throws Exception {
        if (null == connection || connection.isClosed()) {

            Properties connProps = new Properties();
            connProps.setProperty(TSDBDriver.PROPERTY_KEY_CHARSET, "UTF-8");
            connProps.setProperty(TSDBDriver.PROPERTY_KEY_LOCALE, "en_US.UTF-8");
            connProps.setProperty(TSDBDriver.PROPERTY_KEY_TIME_ZONE, "UTC-8");
            connection = DriverManager.getConnection(getJdbcUrl(), connProps);
        }

        return connection;
    }

    public Statement getStatement() throws Exception {
        return getConnectionPool().getConnection().createStatement();
    }
    /**
     * 建表用的链接，没有数据库
     *
     * @return
     * @throws Exception
     */
    public Connection getCreateCon() throws Exception {
        if (null == createCon || createCon.isClosed()) {
            Class.forName(driverType);
            String jdbcUrl = "jdbc:TAOS-RS://" + serverIp + ":" + serverPort + "?user=" + userName + "&password=" + password;
            if (driverType.equals("com.taosdata.jdbc.TSDBDriver"))
                jdbcUrl = "jdbc:TAOS://" + serverIp + ":" + serverPort + "?user=" + userName + "&password=" + password;
            Properties connProps = new Properties();
            connProps.setProperty(TSDBDriver.PROPERTY_KEY_CHARSET, "UTF-8");
            connProps.setProperty(TSDBDriver.PROPERTY_KEY_LOCALE, "en_US.UTF-8");
            connProps.setProperty(TSDBDriver.PROPERTY_KEY_TIME_ZONE, "UTC-8");
            createCon = DriverManager.getConnection(jdbcUrl, connProps);
        }
        return createCon;
    }

    private String getJdbcUrl() throws ClassNotFoundException {

        Class.forName(driverType);
        String mydbname = TaosDatabaseInfo.map.get("mydbname");
        if (StringUtil.isBlank(mydbname)) mydbname = dbName;

        String jdbcUrl = "jdbc:TAOS-RS://" + serverIp + ":" + serverPort + "/" + mydbname + "?user=" + userName + "&password=" + password;
        if (driverType.equals("com.taosdata.jdbc.TSDBDriver"))
            jdbcUrl = "jdbc:TAOS://" + serverIp + ":" + serverPort + "/" + mydbname + "?user=" + userName + "&password=" + password;
        return jdbcUrl;
    }

    public String getTableName(String tagName) {
        return dbName + "." + tagName;
    }

    public String getDbName() {
        return dbName;
    }


    public String getAIStableName() {
        return aiStableName;
    }

    public String getDIStableName() {
        return diStableName;
    }

    private DruidDataSource getConnectionPool() throws ClassNotFoundException {
        if (dataSource == null) {
            dataSource = new DruidDataSource();

            dataSource.setDriverClassName(driverType);
            dataSource.setUrl(getJdbcUrl());
            dataSource.setUsername(userName);
            dataSource.setPassword(password);
            // pool configurations
            dataSource.setInitialSize(10);
            dataSource.setMinIdle(10);
            dataSource.setMaxActive(10);
            dataSource.setMaxWait(30000);
            dataSource.setValidationQuery("select server_status()");
        }
        return dataSource;
    }
}