package com.gyee.redis2taos.controller;

import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import com.alibaba.fastjson.JSONObject;
import com.gyee.redis2taos.config.FilePathConfig;
import com.gyee.redis2taos.entity.TaosEntity;
import com.gyee.redis2taos.mapper.PointAiMapper;
import com.gyee.redis2taos.mapper.PointDiMapper;
import com.gyee.redis2taos.result.Result;
import com.gyee.redis2taos.result.ResultCode;
import com.gyee.redis2taos.service.FileService;
import com.gyee.redis2taos.service.ReadAndWriteService;
import com.gyee.redis2taos.service.RedisService;
import com.gyee.redis2taos.service.TaosService;
import com.gyee.redis2taos.util.FileUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.util.IOUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@RestController()
@RequestMapping("/taos")
public class TaosController {

    @Resource
    private RedisService redisService;
    @Resource
    private TaosService taosService;
    @Resource
    private ReadAndWriteService readAndWriteService;
    @Resource
    private PointAiMapper pointAiMapper;
    @Resource
    private PointDiMapper pointDiMapper;
    @Resource
    private FileService fileService;

    //TODO 升级用mybatis-plus

    @GetMapping("/create/database")
    private JSONObject stableCreat(@RequestParam("dbname") String dbname, @RequestParam("keepday") int keepday) {
        try {
            taosService.createDataBase(dbname, keepday);
            return Result.seccess("数据库创建成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error();
        }
    }

    @GetMapping("/create/supertable")
    private JSONObject stableCreat(@RequestParam(value = "aiSTablename", required = false) String aiSTablename,
                                   @RequestParam(value = "diSTablename", required = false) String diSTablename) {
        try {
            boolean b = taosService.createStables(aiSTablename, diSTablename);
            if (b) return Result.seccess("超级表创建成功！");
            return Result.error();
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error();
        }
    }

    @GetMapping("/create/table")
    private JSONObject tableCreat() {
        try {
            int tables = taosService.createTables();
            if (tables > 0) Result.seccess("点表创建成功！");
            return Result.error();
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error();
        }
    }

    @GetMapping("/table/create2")
    private void tableCreat2() {
        try {
            String excelPath = FilePathConfig.getFilePath() + File.separator + "Taos更新表.xlsx";
            ExcelReader reader = ExcelUtil.getReader(excelPath);
            List<TaosEntity> read = reader.read(1, 2, TaosEntity.class);
            redisService.creatTables(read);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @GetMapping("/table/delete")
    private void tableDelete() {

    }

    @GetMapping("/table/update")
    private void tableUpdate() {

    }

    @GetMapping("/table/select")
    private void tableSelect() {

    }

    @GetMapping("/2redis/start")
    private void taos2RedisStart() throws Exception {
        readAndWriteService.start();
    }

    @GetMapping("/2redis/stop")
    private void taos2RedisStop() {
        readAndWriteService.stop();
    }

    @GetMapping("/redis/updatekeys")
    private void updateRedisKeys() {
        readAndWriteService.getKeys();
    }

    @GetMapping("/template")
    private void getTemplate(HttpServletResponse response) {
        FileUtils.download(FilePathConfig.getFilePath() + File.separator + "Taos更新模板.xlsx", response);
    }

    @PostMapping("/uploadexcel")
    private JSONObject uploadExcel(@RequestBody MultipartFile excelFile) {
        List<TaosEntity> entities = new ArrayList<>();
        //IOUtils.setByteArrayMaxOverride(500000000);

        try {
            FileUtils.write2File(excelFile.getInputStream());
            entities = fileService.getTaosEntitys();
            //ExcelReader reader = ExcelUtil.getReader(FilePathConfig.getTaosInfoTablePath());
            //entities = reader.read(1, 2, TaosEntity.class);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("excel读取失败!");
            return Result.error(4000,"excel读取失败!");
        }

        entities = entities.stream().map(te -> {
            te.setPoint(te.getPoint().replaceFirst("\\.", "_"));
            return te;
        }).collect(Collectors.toList());
        //redisService.creatTables(entities);

        try {
            taosService.createTablesByExcel(entities);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error(4000,"建表或者修改Tag失败！");
        }
        return Result.success();
    }

    @GetMapping("/2redis/status")
    private JSONObject taos2RedisStatus() {
        return Result.successData(readAndWriteService.isActive());
    }
}
