package com.gyee.redis2taos.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gyee.redis2taos.config.TaosConfig;
import com.gyee.redis2taos.entity.Point;
import com.gyee.redis2taos.mapper.PointMapper;
import com.gyee.redis2taos.result.Result;
import com.gyee.redis2taos.result.ResultCode;
import com.gyee.redis2taos.util.TaosDatabaseInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.sql.Statement;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/taoselect")
public class TaosSelectController {

    @Resource
    private TaosConfig taosConfig;
    @Resource
    private PointMapper pointMapper;

    @GetMapping("/stations")
    private JSONObject getStations(){
        /*String sql = null;
        try {
            taosConfig.getStatement().executeQuery(sql);
        } catch (Exception e) {
            e.printStackTrace();
        }*/
        List<String> strings = pointMapper.selectStation(TaosDatabaseInfo.map.get("aistablename"));
        return Result.successData(strings);
    }

    @GetMapping("/title")
    private JSONObject getTitle(){
        Map<String, String> map = new LinkedHashMap<>();
        map.put("dbnme", TaosDatabaseInfo.map.get("mydbname"));
        map.put("aistablename", TaosDatabaseInfo.map.get("aistablename"));
        map.put("distablename", TaosDatabaseInfo.map.get("distablename"));
        return Result.successData(map);
    }

    @GetMapping("/tables")
    private JSONObject getTables(@RequestParam("stablename") String stablename,
                                 @RequestParam(value = "station",required = false) String station,
                                 @RequestParam(value = "point",required = false) String point,
                                 @RequestParam(value = "description",required = false) String description,
                                 @RequestParam(value = "uniformcode",required = false) String uniformcode,
                                 @RequestParam(value = "pageNum",required = false) Integer pageNum,
                                 @RequestParam(value = "pageSize",required = false) Integer pageSize){
        if (pageNum == null || pageSize == null) {
            pageNum = 1;
            pageSize = 100;
        }
        Page<Point> page = new Page<>(pageNum, pageSize);

        QueryWrapper wrapper = new QueryWrapper<>();
        wrapper.eq(station != null, "station", station)
                .like(point != null, "tbname", point)
                .like(description != null, "description", description)
                .eq(uniformcode != null, "uniformcode", uniformcode);

        List<Point> points = null;
        try{
            points = pointMapper.selectTableNamesPage(page,stablename,wrapper);
        }catch (Exception e){
            e.printStackTrace();
            return Result.error(ResultCode.ERROR_DATA);
        }

        return Result.successData(points);
    }

    @DeleteMapping("/tables")
    private JSONObject deleteTables(String tablenames){
        String[] split = tablenames.split(",");
        String s = "";
        try {
            for (String ss : split) {
                s = ss;
                pointMapper.deleteTable(ss);
            }
        }catch (Exception e){
            e.printStackTrace();
            log.info(s+"删除出错！");
            return Result.error(4000, s + "删除出错！");
        }

        return Result.success();
    }

    @GetMapping("/table/info")
    private JSONObject getTableInfo(@RequestParam("tablename") String tablename,
                                    @RequestParam(value = "pageNum",required = false) Integer pageNum,
                                    @RequestParam(value = "pageSize",required = false) Integer pageSize,
                                    @RequestParam("startTime") Long startTime,
                                    @RequestParam("endTime") Long endTime){
        if (pageNum == null || pageSize == null) {
            pageNum = 1;
            pageSize = 100;
        }
        Page<Point> page = new Page<>(pageNum, pageSize);

        QueryWrapper wrapper = new QueryWrapper<>();
        wrapper.between("ts", startTime, endTime);

        List<Point> points = null;
        try{
            points = pointMapper.selectTableValPage(page, tablename, wrapper);
        }catch (Exception e){
            e.printStackTrace();
            return Result.error(ResultCode.ERROR_DATA);
        }

        return Result.successData(points);
    }
}
