package com.gyee.redis2taos.service;

import cn.hutool.core.collection.ListUtil;
import com.gyee.redis2taos.config.RedisConfig;
import com.gyee.redis2taos.config.TaosConfig;
import com.gyee.redis2taos.timeseries.BasicTsPoint;
import com.gyee.redis2taos.timeseries.TsDataType;
import com.gyee.redis2taos.timeseries.TsPoint;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.boot.DefaultApplicationArguments;
import org.springframework.core.annotation.Order;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Order(1)
@Service
public class ReadAndWriteService implements ApplicationRunner {

    @Resource
    private RedisService redisService;
    @Resource
    private ThreadPoolTaskExecutor taskExecutor;
    @Resource
    private RedisConfig redisConfig;
    @Resource
    private TaosConfig taosConfig;
    @Resource
    private TaosService taosService;

    private List<List<String>> boolStrll = new ArrayList<>();
    private List<List<String>> douStrll = new ArrayList<>();

    @Value("${active:false}")
    private boolean active;

    @Override
    public void run(ApplicationArguments args) {

        getKeys();
        try {
            taosService.init();
        } catch (Exception e) {
            e.printStackTrace();
        }

        while (active) {
            try {
                for (List<String> strings : boolStrll) {
                    taskExecutor.execute(new TaskRedis2Taos(strings, redisConfig, taosConfig, TsDataType.BOOLEAN, taskExecutor));
                }
                for (List<String> strings : douStrll) {
                    taskExecutor.execute(new TaskRedis2Taos(strings, redisConfig, taosConfig, TsDataType.DOUBLE, taskExecutor));
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            log.info("\n#");
        }
    }

    public void getKeys() {
        //long l1 = System.currentTimeMillis();
        Set<String> keys = redisService.getKeys();
        if (keys.size() == 0) return;
        //long l2 = System.currentTimeMillis();
        log.info("读取rediskeys成功！！！," + keys.size() + "个点。");
        //按类型分点
        Map<TsDataType, List<BasicTsPoint>> point = keys.stream().map(ss -> {
            if (ss.contains("DI") || ss.contains("di")) {
                return new BasicTsPoint(ss, TsDataType.BOOLEAN);
            } else {
                return new BasicTsPoint(ss, TsDataType.DOUBLE);
            }
        }).collect(Collectors.groupingBy(TsPoint::getTsDataType));

        List<BasicTsPoint> dipl = point.get(TsDataType.BOOLEAN);
        List<BasicTsPoint> aipl = point.get(TsDataType.DOUBLE);

        //di点
        if (dipl != null) {
            List<String> boolCollect = dipl.stream().map(k1 -> k1.getId()).collect(Collectors.toList());
            //按数量分组，di点，分为9216个1组，redis单机可以控制在1s内
            boolStrll = ListUtil.split(boolCollect, 9216);
        }
        //ai点
        if (aipl != null) {
            List<String> douCollect = aipl.stream().map(k1 -> k1.getId()).collect(Collectors.toList());
            //按数量分组，ai点
            douStrll = ListUtil.split(douCollect, 9216);
        }

    }

    public void stop() {
        this.active = false;
    }

    public void start() throws Exception {
        this.active = true;
        run(new DefaultApplicationArguments());
    }

    public boolean isActive() {
        return active;
    }
}
